/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function Logger(){

	var file;

	this.init = function(filename){
		file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
		file.append("cl_index");
		file.append("log");
		file.append(filename);


		if(!file.exists()){
            file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0666);
        }
	};

	this.log = function(value){
		if(file){

			// file is nsIFile, data is a string
			var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                         	   .createInstance(Components.interfaces.nsIFileOutputStream);

			// use 0x02 | 0x10 to open file for appending.
			foStream.init(file, 0x02 | 0x08 | 0x10, 0666, 0); // write, create, truncate

			value += "\n";
			
			foStream.write(value, value.length);
			foStream.close();
		}
	};

}
 
// TODO:: insert this instead of strings
const Observer_Key_ABORT = 0;
const Observer_Key_NEXT = 1;
const Observer_Key_DONE = 2;
const Observer_Key_ERROR = 3;
const Observer_Key_DUPLICATE = 4;
const Observer_Key_UPDATE = 5;
 
function HTTPPackage(){
	this.resource = null;
	this.currentUrl = "";
	this.tries = -1;
	this.causedErrors = false;
}

/*
	Web crawler based on XMLHTTPRequest.
*/
var Crawler = {

	bookmarks : null,
	indexedBookmarks : null,
	observers : null,
	
	isAborted : false,
	timerID : null,
	httpRequest : null,
	
	urlContainer : null,
	contentContainer : null,
	
	current : 0,
	max : 0,
	errorCount : 0,
	duplicateCount : 0,
	
	check : false,
	alreadyIndexed : false,
	
	logger : null,
	
	init : function(){
		Parser.init();
		
		this.observers = [];
		this.urlContainer = [];
		this.contentContainer = [];
		
		this.current = 0;
		this.max = 0;
		this.errorCount = 0;
		this.duplicateCount = 0;
		
		this.logger = new Logger();
		this.logger.init("CrawlerLog.log");
	},
	
	_init : function(){
		Parser._init();
	
		this.isAborted = false;
		
		this.observers = null;
		this.urlContainer = null;
		this.contentContainer = null;
		
		this.current = 0;
		this.max = 0;
		this.errorCount = 0;
		this.duplicateCount = 0;
		
		this.timerID = null;
		this.httpRequest = null;
		this.check = false;
	},
	
	setBookmarks : function(bookmarks){
		this.bookmarks = bookmarks;
		this.max += this.bookmarks.length;
	},
	
	setIndexedBookmarks : function(bookmarks){
		this.indexedBookmarks = bookmarks;
		this.max += this.bookmarks.length;
	},

	setCheck : function(check){
		this.check = check;
	},
	
	setObserver : function(observer){
		this.observers[observer.key] = observer;
	},

	abortCrawling : function(){

		if(Crawler.httpRequest){
			Crawler.httpRequest.abort();
			Crawler.httpRequest.httpRequest = null;
			//TODO: callback to index
			//writeBlacklist(data.resource, "timeout", "timeout");

			Crawler.buildProcess();
		}
	},
		
	stopTimer : function(){
		if(Crawler.timerID)
			clearTimeout(Crawler.timerID);
	},

	_abort : function(){
		this.isAborted = true;
	},

	_build : function(){
		this.buildProcess();
	},
 
	isUrl : function(s) {
		var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/
		return regexp.test(s);
	},

 
	buildProcess : function(){
		if(!this.isAborted){
			var _package = new HTTPPackage();
			
			_package.resource = this.bookmarks.shift();
			this.current++;
		
			if(_package.resource){
				
				_package.currentUrl = Book.getURI(_package.resource);
				//http://kuh.inf.fu-berlin.de:32768/leistverwalt/
				
				this.innerLoop(_package);
			
			}else {
				if(this.observers[Observer_Key_DONE])
					this.observers[Observer_Key_DONE].observe();
				
				//TODO: check without _init()
				this._init();
			}
		}else {
			if(this.observers[Observer_Key_ABORT])
				this.observers[Observer_Key_ABORT].observe();
			
			this._init();
		}
	},
		 
	innerLoop : function(_package){
		//request failed after 3 tries --> log current resource and write down unknown error
		if(_package.tries == 3){
			this.putError(_package.resource, "unknown", "unknown");
			
			this.httpRequest.abort();
			this.httpRequest = null;
			
			this.buildProcess();
		}else {
			if(!this.failure(_package)){ //request was successfully -> go on with next resource

				this.logger.log("[" + this.current + "]["  + _package.tries + " tries] " + _package.currentUrl + "  success");
				this.handleResponse(_package);
				
			}else {
				_package.tries++;
				
				this.sendHTTPRequest(_package);
			}
		}
	},

	failure : function(_package){
		return(!this.httpRequest || !this.httpRequest.responseText ||
		!this.httpRequest.status || !this.httpRequest.statusText ||
		_package.causedErrors);
	},
	
	putError : function(resource, status, statusText){
		var url = Book.getURI(resource);
	
		var prefs = getPrefs();
		prefs.setCharPref("bookmarkManagement.invBm" + this.errorCount++, url + ";" + status + ";" + statusText);
	
		if(this.observers[Observer_Key_ERROR])
			this.observers[Observer_Key_ERROR].observe(resource, status, statusText);
	},
	
	putDuplicate : function(url){
		var prefs = getPrefs();
	
		prefs.setCharPref("bookmarkManagement.dupBm" + this.duplicateCount++, url);
		
		if(this.observers[Observer_Key_DUPLICATE])
			this.observers[Observer_Key_DUPLICATE].observe(this.duplicateCount);
	},
	
	sendHTTPRequest : function(_package){
	
		if(!this.isUrl(_package.currentUrl)){
			this.logger.log("ERROR: " + _package.currentUrl + " is not a valid url!");
			this.buildProcess();
		}else {
			_package.causedErrors = false;
			this.httpRequest = new XMLHttpRequest();
			this.httpRequest.mozBackgroundRequest = true;
			
			this.timerID = setTimeout(this.abortCrawling, 10000);

			// <request, url, true = async, false = sync>
			this.httpRequest.open("GET", _package.currentUrl, true);
			
			if(this.alreadyIndexed){
				//If-Modified-Since ->> 304 = not modified
				//var d = new Date().getTime() - 604800000;
				//d = new Date(d);
				
				//this.httpRequest.setRequestHeader("If-Modified-Since", d.toGMTString());
			}
			
			this.httpRequest.onerror = function(e){
				_package.causedErrors = true;
			};
			
			this.httpRequest.onreadystatechange = function(e){
				if(Crawler.httpRequest.readyState == 4){ // 4 = COMPLETED	
				
					Crawler.stopTimer(); //abort timeout counter
					
					if(this.alreadyIndexed){ //already indexed
						if(Crawler.httpRequest.status == 304){ //not modified
							this.buildProcess();
							return;
						}
					}
					
					Crawler.innerLoop(_package);
				}
			};

			this.httpRequest.send(null);
		}
	},

	handleResponse : function(_package){
		switch(this.httpRequest.status){
			//OK
			case 200 : {
				this.handleContentType(_package, Parser.getType(this.httpRequest.getResponseHeader("Content-Type")));
				break;
			}
			//Created
			case 201 : {
				this.notImplemented(this.httpRequest.status, _package.currentUrl);
				break;
			}
			//Accepted
			case 202 : {
				this.notImplemented(this.httpRequest.status, _package.currentUrl);
				break;
			}
			//Non-Authoritative Information
			case 203 : {
				this.handleContentType(_package, Parser.getType(this.httpRequest.getResponseHeader("Content-Type")));
				break;
			}
			//No Content
			case 204 : {
				this.notImplemented(this.httpRequest.status, _package.currentUrl);
				break;
			}
			//Reset Content
			case 205 : {
				this.handleContentType(_package, Parser.getType(this.httpRequest.getResponseHeader("Content-Type")));
				break;
			}
			//Partial Content
			case 206 : {
				this.notImplemented(this.httpRequest.status, _package.currentUrl);
				break;
			}
			//Multiple Choices
			case 300 : {
				this.httpRequest.abort();
				this.httpRequest = null;
				
				this.buildProcess();
				break;
			}
			//Moved Permanently
			case 301 : {
				this.redirect(_package);
				break;
			}
			//Found
			case 302 : {
				this.redirect(_package);
				break;
			}
			//See Other
			case 303 : {
				this.redirect(_package);
				break;
			}
			//Not Modified
			case 304 : {
				this.notImplemented(this.httpRequest.status, _package.currentUrl);
				break;
			}
			//Use Proxy
			case 305 : {
				this.redirect(_package);
				break;
			}
			//Temporary Redirect
			case 307 : {
				this.redirect(_package);
				break;
			}
			default : {
				// catch errors
				if(this.httpRequest.status >= 400 && this.httpRequest.status < 600){
					this.putError(_package.resource, this.httpRequest.status, this.httpRequest.statusText);
				}

				this.httpRequest.abort();
				this.httpRequest = null;
				
				this.buildProcess();
				break;
			}
		}
	},
		 
	redirect : function(_package){
		this.logger.log("redirect (" + _package.currentUrl + ") is not implemented.");
		this.httpRequest.abort();
		this.httpRequest = null;
		this.buildProcess();
	},
		 
	notImplemented : function(status, url){
		this.logger.log("http status: " + status + "(" + url + ") is not implemented.");
		this.httpRequest.abort();
		this.httpRequest = null;
		
		this.buildProcess();
	},

	handleContentType : function(_package, type){
		switch(type){
			case "text/html" : {
				this.finalize(_package); break;
			}
			case "text/xml" : {
				this.finalize(_package); break;
			}
			case "text/plain" : {
				// build index with plain text
				break;
			}
			case "application/pdf" : {
				//TODO: implement
				this.httpRequest.abort();
				this.httpRequest = null;
				
				this.buildProcess(); break;
			}
			case "application/xml" : {
				this.finalize(_package); break;
			}
			case "application/xhtml+xml" : {
				this.finalize(_package); break;
			}
			case "application/rss+xml" : {
				this.finalize(_package); break;
			}
			default : { // unknowns type
				this.httpRequest.abort();
				this.httpRequest = null;
				
				this.buildProcess(); break;
			}
		}
	},

	finalize : function(_package){

		var cl_document = Parser.parse(_package.resource, this.httpRequest.responseText);
		
		if(this.contentContainer[cl_document.content] != 2 && this.urlContainer[cl_document.primaryKey] != 2){
			if(!this.check){
				
				//append bookmark properties
				cl_document.content += " ";
				cl_document.content += Book.getName(_package.resource);
				cl_document.content += " ";
				cl_document.content += Book.getURI(_package.resource);
				cl_document.content += " ";
				cl_document.content += Book.getDesc(_package.resource);
				cl_document.content += " ";
				cl_document.content += Book.getKeys(_package.resource);
				
				if(this.alreadyIndexed){
					this.observers[Observer_Key_UPDATE].observe(cl_document, this);
				}else {
					this.observers[Observer_Key_NEXT].observe(cl_document, this);
				}
			}else {
				Index.setProgress(this.current, this.max, _package.currentUrl);
			}
			this.contentContainer[cl_document.content] = 2;
			this.urlContainer[cl_document.primaryKey] = 2;
		}else {
			//write duplicate
			this.putDuplicate(cl_document.primaryKey);
		}
		
		this.httpRequest.abort();
		this.httpRequest = null;

		this.buildProcess();
	}
	
};

var Parser = {

	tidy : null,

	init : function(){
		var instTidy = Components.classes["@bookmarktools.mozdev.org/repair/tidy;1"].createInstance();
		this.tidy = instTidy.QueryInterface(Components.interfaces.ITidy);
	},
	
	_init : function(){
		this.tidy = null;
	},

	removeScript : function(markup){
		return markup.replace(/<script([\r\n]|.)*?>([\r\n]|.)*?<\/script>/gi,"");
	},

	removeTags : function(content){
		return content.replace(/(<([^>]+)>)/g,""); 
	},

	removeWhiteSpaces : function(content){
		return content.replace(/(\s)+/gi," ");
	},
	
	getType : function(str){
		if(str && str != null){
			if(str.indexOf("text/html") != -1) return "text/html";
			if(str.indexOf("text/xml") != -1) return "text/xml";
			if(str.indexOf("text/plain") != -1) return "text/plain";
			if(str.indexOf("application/pdf") != -1) return "application/pdf";
			if(str.indexOf("application/xml") != -1) return "application/xml";
			if(str.indexOf("application/xhtml+xml") != -1) return "application/xhtml+xml";
			if(str.indexOf("application/rss+xml") != -1) return "application/rss+xml";
		}
		return "unknown";
	},
	
	getMCList : function(response){
		response = this.tidy.cleanup(response);

		var urls = response.match(/href=\".*\"/gi);
		if(urls != null){
			var href = response.match(/href=/gi);

			var size = urls.length;
			for(var i = 0 ;i < size; i++){
				urls[i] = urls[i].replace(href, "").replace('"', "");
				urls[i] = urls[i].replace(href, "").replace('"', "");
			}
		}

		return urls;
	},

	getSplitedURL : function(str){
		var splited = str.split("/");
		return splited[0] + "//" + splited[2];
	},
	
	parse : function(resource, content){

		//remove script entries
		var response = this.removeScript(content);
		var wellformed = true;

		//define clucene document
		var cl_document = new Object();

		// if the document is wellformed return the content
		response = this.tidy.cleanup(response);

		if(response == "Tidy: no output")
			wellformed = false;

		cl_document.primaryKey = Book.getURI(resource);
		cl_document.name = Book.getName(resource);
		
		cl_document.content = wellformed ? this.removeWhiteSpaces(this.removeTags(response)) : content;
		
		return cl_document;
	}

};