/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//sort order, begins with descending
var order;
const pref_error = "bookmarkManagement.errorShow";

//+-------------------------------------------------
//| helper functions
//+-------------------------------------------------

function doSort(){

	var tree_error = document.getElementById("ERRORTREE");

	var field = new Array();
	var size = tree_error.size;
	for(var i = 0 ;i < size; i++){
		var item = tree_error.getItem(i);
		var cell = item.childNodes[0].childNodes[1];
		var error = cell.getAttribute("value");

		var obj = new Object();
		obj.value = error;
		obj.item = item;
		field.push(obj);
	}

	if(order == 0){
		order = 1;
		tree_error.getColumn(1).setAttribute("sortDirection", "ascending");
	}else {
		order = 0;
		tree_error.getColumn(1).setAttribute("sortDirection", "descending");
	}

	qsort(0, field.length - 1, field, order);

	tree_error.clear();

	size = field.length;
	for(var i = 0 ;i < size; i++){
		tree_error.addItem(field[i].item);
	}

}

function doErrorShow(range){
	var tree_error = document.getElementById("ERRORTREE");
	tree_error.clear();
	
	var prefs = getPrefs();
	var errors = getErrors();

	var resources = Book.getAllResourcesHash();
	
	var s = errors.length;
	for(var i = 0 ;i < s; i++){
		var key = "bookmarkManagement.invBm" + errors[i];

		var tmp = prefs.getCharPref(key).split(";");
		var url = tmp[0];
		var error = tmp[1];
		var status = tmp[2];

		var resource = resources[url];
		
		switch(range){
			case 0 : { //all
				if(resource){
					tree_error.add(Book.getName(resource), Book.getURI(resource), error, status);
				}
				break;
			}
			case 1 : { //5xx
				if(resource && error >= 500){
					tree_error.add(Book.getName(resource), Book.getURI(resource), error, status);
				}
				break;
			}
			case 2 : { //4xx
				if(resource && error >= 400 && error < 500){
					tree_error.add(Book.getName(resource), Book.getURI(resource), error, status);
				}
				break;
			}
			case 3 : { //unknown
				if(resource && error == "unknown"){
					 tree_error.add(Book.getName(resource), Book.getURI(resource), error, status);
				}
				break;
			}
		}
	}

	document.getElementById("ERROR_COUNT").value = "(" + tree_error.size + ")";
	tree_error.getColumn(1).setAttribute("sortDirection", "none");
}

function doDiscard(){
	var tree_error = document.getElementById("ERRORTREE");
	var items = tree_error.selection;

	var resources = Book.getAllResourcesHash();

	tree_error.unselect();

	var prefs = getPrefs();
	
	var size = items.length;
	for(var i = size - 1 ;i >= 0; i--){

		var url = tree_error.getItemURL(items[i]);

		if(resources[url]){
			Book.removeBookmark(resources[url].itemId);
		}

		tree_error.remove(items[i]); 

		var errors = getErrors();
		var s = errors.length;
		for(var j = 0 ;j < s; j++){
			var key = "bookmarkManagement.invBm" + errors[j];
			var url2 = prefs.getCharPref(key);
			
			if(url2 == url)
				prefs.clearUserPref(key);
		}		
	}
	
	document.getElementById("ERROR_COUNT").value = "(" + tree_error.size + ")";
	tree_error.getColumn(1).setAttribute("sortDirection", "none");
}

//+-------------------------------------------------
//| load & save functions
//+-------------------------------------------------

function onErrorLoad(){
	var prefs = getPrefs();
	var range = prefs.getIntPref(pref_error);
	document.getElementById("ERROR_SHOW").selectedIndex = range;
	
	doErrorShow(range);
}
