/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


/*
	Basic indexer that wraps the clucene component into the js world.	
*/
var Index = {

	path : "",
	delay : 604800000, //default delay one week (in millis)
	crawler : null,
	view : null,
	win : null,
	
	indexWriter : null,
	indexReader : null,
	
	init : function(){
		var instWriter = Components.classes["@bookmarktools.mozdev.org/search/indexWriter;1"].createInstance();
		this.indexWriter = instWriter.QueryInterface(Components.interfaces.ICLWriter);
			
		var instReader = Components.classes["@bookmarktools.mozdev.org/search/indexReader;1"].createInstance();
		this.indexReader = instReader.QueryInterface(Components.interfaces.ICLReader);

		this.indexReader.setPath(getSearchDir().path);
	},
	
	//-------------------------------------------------------------------------------
	//	Define setter
	//-------------------------------------------------------------------------------
	
	setPath : function(isAuto){
		this.path = isAuto ? getBuildDir().path : getSearchDir().path;
		this.indexWriter.setPath(this.path);
	},
	
	setDelay : function(delay){
		this.delay = delay;
	},
	
	setView : function(view){
		this.view = view;
	},
	
	setWindow : function(win){
		this.win = win;
	},
	
	setCrawler : function(crawler){
		this.crawler = crawler;
		
		var Observer = {
			key : Observer_Key_NEXT,
			observe : function(cl_document, caller){
				Index.indexWriter.appendDocument(
					cl_document.primaryKey,
					cl_document.name,
					cl_document.content
				);
				Index.setProgress(caller.current, caller.max, cl_document.primaryKey);
			}
		};
		
		this.crawler.setObserver(Observer);
	},
	
	initGUI : function(){
		this.view.getElementById("PROGRESS_LABEL").value = "0%";
		this.view.getElementById("PROGRESS").value = 0;
	},

	stopGUI : function(){
		this.view.getElementById("PROGRESS").value = 100;
		this.view.getElementById("PROGRESS_LABEL").value = "100%";
		
		if(this.win != null) this.win.close();
	},
	
	setProgress : function(current, max, url){
		this.view.getElementById("CURRENT_URL").value = "[" + current + "/" + max + "] " + (url.length > 30 ? url.substring(0, 29) + "[..]" : url);
		this.view.getElementById("PROGRESS_LABEL").value = Math.floor((current / max) * 100) + "%";
		this.view.getElementById("PROGRESS").value = (current / max) * 100;
	},
	
	//-------------------------------------------------------------------------------
	//	Define main functions (_build, _search, _delete, _lookup, _abort)
	//-------------------------------------------------------------------------------

	// constructs index data (date = 0 ==> start build, otherwise check against last build date)
	_build : function(date){
		if(this.crawler){
			var go = true;
			if(date != 0){
				if((date + this.delay) >= new Date().getTime()){
					go = false;
				}
			}
			
			if(go){
				this.initGUI();
				this.crawler._build();
			}
		}else {
			//TODO log
			//alert("No crawler available!");
		}
	},

	// find pattern and update the given searchTree instance
	_search : function(pattern, searchTree, chunk, chunkSize){
		var max = 0;
		if(this.indexReader.exists()){ //index exists ?
			// validate pattern
			setAlertUser(false);
			var query = !doCheckLuceneQueryValue(pattern) ? "" : pattern;

			if(query.length > 2){

				var array = Components.classes["@mozilla.org/array;1"]
							.createInstance(Components.interfaces.nsIMutableArray);
				max = this.indexReader.search(query, array, chunk, chunkSize);

				//clear result tree
				searchTree.clear();

				var resources = Book.getAllResourcesHash();
				
				//draw results
				var e = array.enumerate();
				while (e.hasMoreElements()) {
					var resultNode = e.getNext().QueryInterface(Components.interfaces.IResultNode);
					var node = resources[resultNode.primaryKey];
					if(node)
						searchTree.add(Book.getName(node), resultNode.primaryKey, resultNode.score);
				}
			}else {
				searchTree.clear();
			}
		}else {
			alert("Error: No index data found!");
		}
		return max;
	},
	
	_searchSidebar : function(pattern, sidebar, chunk, chunkSize){
		var max = 0;
		if(this.indexReader.exists()){ //index exists ?
			// validate pattern
			setAlertUser(false);
			var query = !doCheckLuceneQueryValue(pattern) ? "" : pattern;
		
			if(query.length > 2){

				var array = Components.classes["@mozilla.org/array;1"]
							.createInstance(Components.interfaces.nsIMutableArray);
				max = this.indexReader.search(query, array, chunk, chunkSize);

				//clear result tree
				sidebar.clear();
				
				var resources = Book.getAllResourcesHash();
				
				//draw results
				var e = array.enumerate();
				while (e.hasMoreElements()) {
					var resultNode = e.getNext().QueryInterface(Components.interfaces.IResultNode);
					var node = resources[resultNode.primaryKey];
					if(node)
						sidebar.add(Book.getName(node), resultNode.primaryKey, resultNode.score);
				}
			}else {
				sidebar.clear();
			}
		}else {
			alert("Error: No index data found!");
		}
		return max;
	},

	// delete documents
	_delete : function(nodes){
		if(nodes.length != 0){
			this.indexWriter.deleteDocument(nodes, nodes.length);
		}else {
			// it is possible to delete every document but
			//faster to clear the whole directory
			clear(getSearchDir());
		}
	},

	// return the index as mutable array
	_lookup : function(){
		var array = Components.classes["@mozilla.org/array;1"]
                    .createInstance(Components.interfaces.nsIMutableArray);
		this.indexReader.lookup(array);
		return array;
	},
	
	// if build is running this function will abort the crawler
	_abort : function(){
		if(this.crawler){
			this.crawler._abort();
		}else {
			//TODO log
			//alert("No crawler available!");
		}
	}

};

Index.init();
