/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
	Define global helper (Base64, UTF8).
*/

var gInstallRDF = new InstallRDF();
gInstallRDF.parse("maik.gollnick@gmx.de");

/*
	Performs quick sort for a given object array. Each object contains
	a value and item.

	0 descending
	1 ascending
*/
function qsort(l, r, field, order) {
	var i, j, m, x;
	i = l;
	j = r;
	m = field[Math.round((l + r) / 2)];
	do {
		switch(order){
		case 0 : {
			while (field[i].value > m.value) {i++};
			while (m.value > field[j].value) {j--};
			break;
		}
		case 1 : {
			while (field[i].value < m.value) {i++};
			while (m.value < field[j].value) {j--};
			break;
		}
		}
		if (i <= j) {
			x = field[i];
			field[i] = field[j];
			field[j] = x;
			i++;
			j--;
		}
	} while (i <= j);
	if (l < j) {qsort(l, j, field, order)};
	if (i < r) {qsort(i, r, field, order)};
}

/*
	Returns the build directory.
 */
function getBuildDir(){
	var file = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("ProfD", Components.interfaces.nsIFile);

	file.append("cl_index");
	file.append("build");
	if(!file.exists() || !file.isDirectory()) {
		file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
	}

	return file.clone();
}

/*
	Returns the search directory.
*/
function getSearchDir(){
	var file = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("ProfD", Components.interfaces.nsIFile);

	file.append("cl_index");
	file.append("search");
	if(!file.exists() || !file.isDirectory()) {
		file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
	}

	return file.clone();
}

/*
  Moves all files from the build to the search directory.
*/
function move(bdir, sdir){
	var entries = bdir.directoryEntries;
	while(entries.hasMoreElements()){
	var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
		if(!file.isDirectory()){
			// move file without renaming
			file.moveTo(sdir, file.leafName);
		}
	}
}

/*
	Returns preference object.
*/
function getPrefs(){
	return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefBranch);
}

function clearDirectories(){
	clear(getSearchDir());
	clear(getBuildDir());
}

function clear(dir){
	var entries = dir.directoryEntries;
	while(entries.hasMoreElements()){
		var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
		if(file.isDirectory()){
			file.remove(true);
		}else {
			file.remove(false);
		}
	}
}

function disableDefaultSearch(){
	var prefs = getPrefs();
	if(prefs.getBoolPref("bookmarkManagement.checkDisSe")){
		var defaultKey = document.getElementById("viewBookmarksSidebarKb");
		var defaultKeySet = document.getElementById("mainKeyset");
		defaultKeySet.removeChild(defaultKey);
	}
}

function getURLAsURI (url){
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	return ioService.newURI(url, null, null);
}

function getErrors(){
	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService);
	var branch = prefs.getBranch("bookmarkManagement.invBm");
	return branch.getChildList("", {});
}

function getDuplicates(){
	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService);
	var branch = prefs.getBranch("bookmarkManagement.dupBm");
	return branch.getChildList("", {});
}

function getErrorHash(){
	var prefs = getPrefs();
	
	// clear all error entries
	var dest = [];
	var errors = getErrors();
	for(var i = 0 ;i < errors.length; i++){
		var key = "bookmarkManagement.invBm" + errors[i];
		var entry = prefs.getCharPref(key).split(";");
		dest[entry[0]] = entry[0];
	}
	return dest;
}

function clearPreferences(){
	var prefs = getPrefs();
	
	// clear all error entries
	var errors = getErrors();
	while(errors.length != 0){
		var key = "bookmarkManagement.invBm" + errors[0];
		prefs.clearUserPref(key);
		errors = getErrors();
	}
	
	// clear all duplicate entries
	var duplicates = getDuplicates();
	while(duplicates.length != 0){
		var key = "bookmarkManagement.dupBm" + duplicates[0];
		prefs.clearUserPref(key);
		duplicates = getDuplicates();
	}
}
